﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Diagnostics;
using System.Threading;

using Syntec.Remote;

namespace SyntecRemoteClient
{
	public partial class ExampleForm : Form
	{
		List<SyntecRemoteCNC> m_CNC;
		SyntecRemoteCNC API1 = null;
		System.Windows.Forms.Timer m_tmr300ms;

		public ExampleForm()
		{
			InitializeComponent();
			RemoveUnusedProcess();

			m_tmr300ms = new System.Windows.Forms.Timer();
			m_tmr300ms.Interval = 300;
			m_tmr300ms.Tick += new EventHandler( m_tmr300ms_Tick );
			m_tmr300ms.Enabled = true;
			buttonTimerOn.Enabled = false;
			buttonTimerOff.Enabled = true;

			m_CNC = new List<SyntecRemoteCNC>();

			// Add your CNCs' host-ip here
			SyntecRemoteCNC cnc = new SyntecRemoteCNC( "10.10.80.163" );
			m_CNC.Add( cnc );

			//cnc = new SyntecRemoteCNC( "10.10.1.207" );
			//m_CNC.Add( cnc );
			//cnc = new SyntecRemoteCNC( "10.10.1.96" );
			//m_CNC.Add( cnc );

			foreach( SyntecRemoteCNC tmp in m_CNC ) {
				listBox1.Items.Add( tmp.Host );
			}
		}

		private void m_tmr300ms_Tick( object sender, EventArgs e )
		{
			if( m_CNC.Count >= 1 ) {
				SyntecRemoteCNC cnc = m_CNC[ 0 ];
				short DecPoint = 0;
				string[] AxisName = null, Unit = null;
				float[] Mach = null, Abs = null, Rel = null, Dist = null;

				short result = cnc.READ_position( out AxisName, out DecPoint, out Unit, out Mach, out Abs, out Rel, out Dist );
				if( result == 0 ) {
					if( AxisName.Length > 0 ) {
						label1.Text = AxisName[ 0 ] + " : " + Mach[ 0 ].ToString();
					}
					if( AxisName.Length > 1 ) {
						label2.Text = AxisName[ 1 ] + " : " + Mach[ 1 ].ToString();
					}
					if( AxisName.Length > 2 ) {
						label3.Text = AxisName[ 2 ] + " : " + Mach[ 2 ].ToString();
					}
					if( AxisName.Length > 3 ) {
						label4.Text = AxisName[ 3 ] + " : " + Mach[ 3 ].ToString();
					}
				}
				else {
					label1.Text = "Err : " + result.ToString();
					label2.Text = "Err : " + result.ToString();
					label3.Text = "Err : " + result.ToString();
					label4.Text = "Err : " + result.ToString();
				}
			}
		}
		private void btnTimerOn_Click( object sender, EventArgs e )
		{
			m_tmr300ms.Enabled = true;

			buttonTimerOn.Enabled = false;
			buttonTimerOff.Enabled = true;
		}
		private void btnTimerOff_Click( object sender, EventArgs e )
		{
			m_tmr300ms.Enabled = false;

			buttonTimerOn.Enabled = true;
			buttonTimerOff.Enabled = false;
		}


		~ExampleForm()
		{
			deinit();
			RemoveUnusedProcess();
		}
		private void RemoveUnusedProcess()
		{
			Process[] process = Process.GetProcessesByName( "SyntecRemoteServer" );
			foreach( Process p in process ) {
				p.Kill();
			}
		}
		private void deinit()
		{
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				cnc.Close();
			}
		}

		private void button10_Click( object sender, EventArgs e )
		{
			if( API1.isConnected() ) {

				short Axes = 0, MaxAxes = 0;
				string CncType = "", Series = "", Nc_Ver = "";
				string[] AxisName = null;

				short result = API1.READ_information( out Axes, out CncType, out MaxAxes, out Series, out Nc_Ver, out AxisName );
				if( result < 0 ) {
					MessageBox.Show( "Error" );
				}

				float[] data = new float[ Axes ];

				for( int i = 0; i < Axes; i++ ) {
					data[ i ] = (float)Convert.ToDouble( textBox4.Text );
				}

				result = API1.WRITE_work_coord_single( "EXT", data );
				if( result == 0 ) {
					MessageBox.Show( "Done" );
				}
				else {
					MessageBox.Show( "Error" + result.ToString() );
				}
			}
		}
		private void button13_Click( object sender, EventArgs e )
		{
			if( API1.isConnected() ) {
				if( API1.WRITE_macro_single( 1, Convert.ToDouble( textBox5.Text ) ) == 0 ) {
					MessageBox.Show( "Done" );
				}
				else {
					MessageBox.Show( "Error" );
				}
			}
		}
		private void button24_Click( object sender, EventArgs e )
		{
			if( API1.isConnected() ) {

				string[] title = null;
				if( API1.READ_offset_title( out title ) != 0 ) {
					return;
				}

				listBox3.Items.Clear();
				double[] offset = new double[ title.Length ];

				for( int i = 0; i < offset.Length; i++ ) {
					offset[ i ] = Convert.ToDouble( textBox10.Text ) + 0.001 * i;
				}

				if( API1.WRITE_offset_single( 45, offset ) == 0 ) {
					MessageBox.Show( "Done" );
				}
				else {
					MessageBox.Show( "Error" );
				}
			}
		}
		private void button25_Click( object sender, EventArgs e )
		{
			if( API1.isConnected() ) {

				string[] title = null;
				if( API1.READ_offset_title( out title ) != 0 ) {
					return;
				}

				listBox3.Items.Clear();
				double[][] offset = new double[ 11 ][];

				for( int k = 0; k < 11; k++ ) {
					offset[ k ] = new double[ title.Length ];
					for( int i = 0; i < offset[ k ].Length; i++ ) {
						offset[ k ][ i ] = Convert.ToDouble( textBox11.Text );
					}
				}

				if( API1.WRITE_offset_all( offset ) == 0 ) {
					MessageBox.Show( "Done" );
				}
				else {
					MessageBox.Show( "Error" );
				}
			}
		}

		private void btnREAD_Information_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				short Axes = 0, MaxAxes = 0;
				string CncType = "", Series = "", Nc_Ver = "";
				string[] AxisName = null;

				short result = cnc.READ_information( out Axes, out CncType, out MaxAxes, out Series, out Nc_Ver, out AxisName );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Axes : " + Axes.ToString() );
					listBox3.Items.Add( "CncType : " + CncType.ToString() );
					listBox3.Items.Add( "MaxAxes : " + MaxAxes.ToString() );
					listBox3.Items.Add( "Series : " + Series.ToString() );
					listBox3.Items.Add( "NC_Ver : " + Nc_Ver.ToString() );

					string str = "";
					if( AxisName != null && AxisName.Length > 0 ) {
						str = AxisName[ 0 ];
						for( int i = 1; i < AxisName.Length; i++ ) {
							str += ":" + AxisName[ i ];
						}
					}
					listBox3.Items.Add( "AxisName : " + str );
				}
				else {
					listBox3.Items.Add( cnc.Host + ":Error:testREAD_imformation:" + result.ToString() );
				}
			}
		}
		private void btnREAD_status_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int CurSeq = 0;
				string MainProg = "", CurProg = "", Mode = "", Status = "", Alarm = "", EMG = "";

				short result = cnc.READ_status( out MainProg, out CurProg, out CurSeq, out Mode, out Status, out Alarm, out EMG );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "MainProg : " + MainProg.ToString() );
					listBox3.Items.Add( "CurrentProg : " + CurProg.ToString() );
					listBox3.Items.Add( "CurrentSeq : " + CurSeq.ToString() );
					listBox3.Items.Add( "Mode : " + Mode.ToString() );
					listBox3.Items.Add( "Status : " + Status.ToString() );
					listBox3.Items.Add( "Alarm : " + Alarm.ToString() );
					listBox3.Items.Add( "EMG : " + EMG.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:testREAD_status:" + result.ToString() );
				}
			}
		}
		private void btnREAD_position_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				short DecPoint = 0;
				string[] AxisName = null, Unit = null;
				float[] Mach = null, Abs = null, Rel = null, Dist = null;

				short result = cnc.READ_position( out AxisName, out DecPoint, out Unit, out Mach, out Abs, out Rel, out Dist );
				if( result == 0 ) {
					string str = AxisName[ 0 ];
					for( int i = 1; i < AxisName.Length; i++ ) {
						str += ":" + AxisName[ i ];
					}
					listBox3.Items.Insert( 0, "AxisName : " + str );

					listBox3.Items.Insert( 0, "DecPoint : " + DecPoint.ToString() );

					str = Unit[ 0 ];
					for( int i = 1; i < Unit.Length; i++ ) {
						str += ":" + Unit[ i ];
					}
					listBox3.Items.Insert( 0, "Unit : " + str );

					str = Mach[ 0 ].ToString();
					for( int i = 1; i < Mach.Length; i++ ) {
						str += ":" + Mach[ i ].ToString();
					}
					listBox3.Items.Insert( 0, "Machine : " + str );

					str = Abs[ 0 ].ToString();
					for( int i = 1; i < Abs.Length; i++ ) {
						str += ":" + Abs[ i ].ToString();
					}
					listBox3.Items.Insert( 0, "Absolute : " + str );

					str = Rel[ 0 ].ToString();
					for( int i = 1; i < Rel.Length; i++ ) {
						str += ":" + Rel[ i ].ToString();
					}
					listBox3.Items.Insert( 0, "Relative : " + str );

					str = Dist[ 0 ].ToString();
					for( int i = 1; i < Dist.Length; i++ ) {
						str += ":" + Dist[ i ].ToString();
					}
					listBox3.Items.Insert( 0, "Distance : " + str );

					listBox3.Items.Insert( 0, "===" + cnc.Host + "===" );
				}
				else {
					listBox3.Items.Insert( 0, cnc.Host + " : Error:testREAD_position:" + result.ToString() );
				}
			}
		}
		private void btnREAD_gcode_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();

			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string[] var = null;
				short result = cnc.READ_gcode( out var );
				if( result != (short)Syntec.Remote.SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
				else {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					for( int i = 0; i < var.Length; i++ )
						listBox3.Items.Add( var[ i ] );
				}
			}
		}
		private void btnRead_othercode_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int D = 0, H = 0, S = 0, T = 0, M = 0, F = 0;
				short result;
				if( ( result = cnc.READ_othercode( out H, out D, out T, out M, out F, out S ) ) == (short)Syntec.Remote.SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "H:" + H.ToString() );
					listBox3.Items.Add( "D:" + D.ToString() );
					listBox3.Items.Add( "T:" + T.ToString() );
					listBox3.Items.Add( "M:" + M.ToString() );
					listBox3.Items.Add( "F:" + F.ToString() );
					listBox3.Items.Add( "S:" + S.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnRead_feed_spindle_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				float OvFeed = 0.0F, OvSpindle = 0.0F, ActFeed = 0.0F;
				int ActSpindle = 0;
				short result = cnc.READ_spindle( out OvFeed, out OvSpindle, out ActFeed, out ActSpindle );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Override Feedrate : " + OvFeed.ToString() );
					listBox3.Items.Add( "Override Spindle : " + OvSpindle.ToString() );
					listBox3.Items.Add( "Actual Feedrate : " + ActFeed.ToString() );
					listBox3.Items.Add( "Actual Spindle : " + ActSpindle.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnRead_part_count_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int part_count = 0;
				int require_part_count = 0;
				int total_part_count = 0;
				short result = cnc.READ_part_count( out part_count, out require_part_count, out total_part_count );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "PartCount : " + part_count.ToString() );
					listBox3.Items.Add( "RequirePartCount : " + part_count.ToString() );
					listBox3.Items.Add( "TotalPartCount : " + part_count.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnRead_Time_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int Power = 0, Cutting = 0, Cycle = 0, WorkTime = 0;
				short result = cnc.READ_time( out Power, out Cutting, out Cycle, out WorkTime );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "PowerOnTime : " + Power.ToString() );
					listBox3.Items.Add( "CuttingTime : " + Cutting.ToString() );
					listBox3.Items.Add( "CycleTime" + Cycle.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_alm_current_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				bool isAlarm = false;
				string[] msg = null;
				DateTime[] time = null;
				short result = cnc.READ_alm_current( out isAlarm, out msg, out time );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "HasAlarm : " + isAlarm.ToString() );
					if( msg != null ) {
						for( int i = 0; i < msg.Length; i++ ) {
							listBox3.Items.Add( "Time : " + time[ i ].ToString( "yyyyMMdd HHmmss" ) );
							listBox3.Items.Add( "MSG : " + msg[ i ] );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_alm_history_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string[] msg = null;
				DateTime[] time = null;
				short result = cnc.READ_alm_history( out msg, out time );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( msg != null ) {
						for( int i = 0; i < msg.Length; i++ ) {
							listBox3.Items.Add( "Time : " + time[ i ].ToString( "yyyyMMdd HHmmss" ) );
							listBox3.Items.Add( "MSG : " + msg[ i ] );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_offset_title_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string[] title = null;
				short result = cnc.READ_offset_title( out title );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( title != null ) {
						for( int i = 0; i < title.Length; i++ ) {
							listBox3.Items.Add( title[ i ] );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_offset_all_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				double[][] offset = null;
				short result = cnc.READ_offset_all( out offset );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( offset != null ) {
						for( int i = 0; i < offset.Length; i++ ) {
							string tmp = "";
							for( int k = 0; k < offset[ i ].Length; k++ ) {
								tmp += offset[ i ][ k ].ToString() + " ";
							}
							listBox3.Items.Add( "T" + ( (int)( i + 1 ) ).ToString() + " : " + tmp );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_offset_scope_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				double[][] offset = null;
				short result =cnc.READ_offset_scope( 20, 30, out offset );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( offset != null ) {
						for( int i = 0; i < offset.Length; i++ ) {
							string tmp = "";
							for( int k = 0; k < offset[ i ].Length; k++ ) {
								tmp += offset[ i ][ k ].ToString() + " ";
							}
							listBox3.Items.Add( "T" + ( (int)( i + 20 ) ).ToString() + " : " + tmp );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}

		}
		private void btnREAD_offset_single_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				double[] offset = null;
				short result = cnc.READ_offset_single( 45, out offset );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					string tmp = "";
					if( offset != null ) {
						for( int k = 0; k < offset.Length; k++ ) {
							tmp += offset[ k ].ToString() + " ";
						}
					}
					listBox3.Items.Add( "T45 : " + tmp );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_work_coord_all_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string tmp = "";
				string[] name = null;
				float[][] data = null;
				short result = cnc.READ_work_coord_all( out name, out data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( name != null && data != null ) {
						for( int k = 0; k < name.Length; k++ ) {
							listBox3.Items.Add( name[ k ] );
							tmp = "";
							for( int i = 0; i < data[ 0 ].Length; i++ ) {
								tmp += data[ k ][ i ].ToString() + " | ";
							}
							listBox3.Items.Add( tmp );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_work_coord_axis_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string[] WorkCoordTitle = null;
				string tmp = "";
				short result = cnc.READ_work_coord_axis( out WorkCoordTitle );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( WorkCoordTitle != null ) {
						for( int k = 0; k < WorkCoordTitle.Length; k++ ) {
							tmp += WorkCoordTitle[ k ] + " ";
						}
						listBox3.Items.Add( tmp );
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_work_coord_scope_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string tmp = "";
				string[] name = null;
				float[][] data = null;
				short result = cnc.READ_work_coord_scope( 5, 10, out name, out data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( name != null && data != null ) {
						for( int k = 0; k < name.Length; k++ ) {
							listBox3.Items.Add( name[ k ] );
							tmp = "";
							for( int i = 0; i < data[ 0 ].Length; i++ ) {
								tmp += data[ k ][ i ].ToString() + " | ";
							}
							listBox3.Items.Add( tmp );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_work_coord_single_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				float[] data = null;
				short result = cnc.READ_work_coord_single( "G54", out data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "G54" );
					string tmp = "";
					for( int i = 0; i < data.Length; i++ ) {
						tmp += data[ i ].ToString() + " | ";
					}
					listBox3.Items.Add( tmp );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_work_coord_count_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				short Count = 0;
				short result = cnc.READ_work_coord_count( out Count );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "WorkPiece Count : " + Count.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_macro_all_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int[] Name = null;
				double[] Data = null;
				short result = cnc.READ_macro_all( out Name, out Data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( Name != null && Data != null ) {
						for( int i = 0; i <= 20; i++ ) {
							listBox3.Items.Add( Name[ i ].ToString() +" : "+ Data[ i ].ToString() );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_macro_variable_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int[][] Data = null;
				short result = cnc.READ_macro_variable( out Data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( Data != null ) {
						for( int i = 0; i < Data.Length; i++ ) {
							listBox3.Items.Add( Data[ i ][ 0 ].ToString() + " ~ " + Data[ i ][ 1 ].ToString() );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_macro_scope_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int[] Name = null;
				double[] Data = null;
				short result = cnc.READ_macro_scope( 100, 120, out Name, out Data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( Name != null && Data != null ) {
						for( int i = 0; i <= 20; i++ ) {
							listBox3.Items.Add( Name[ i ].ToString() + " : " + Data[ i ].ToString() );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_macro_single_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				double Data = 0;
				short result = cnc.READ_macro_single( 110, out Data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "110 : " + Data.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}

		}
		private void btnREAD_param_max_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int Data = 0;
				short result = cnc.READ_param_max( out Data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Param MAX : " + Data.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_param_data_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int[] data = null;
				short result = cnc.READ_param_data( 21, 30, out data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( data != null ) {
						for( int i = 0; i < data.Length; i++ ) {
							listBox3.Items.Add( "Param[" + ( (int)( 21 + i ) ).ToString() + "] : " + data[ i ].ToString() );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_plc_addr_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				byte[] plcdataB = null;
				int[] plcdataI = null;
				short[] plcdataS = null;
				short type;
				short result = cnc.READ_plc_addr( "R", 26, 30, out type, out plcdataB, out plcdataS, out plcdataI );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( type == 0 ) {
						if( plcdataB != null ) {
							for( int i = 0; i < plcdataB.Length; i++ ) {
								listBox3.Items.Add( "R" + ( (int)( 26 + i ) ).ToString() +" : "+ plcdataB[ i ].ToString() );
							}
						}
					}
					else if( type == 2 ) {
						if( plcdataI != null ) {
							for( int i = 0; i < plcdataI.Length; i++ ) {
								listBox3.Items.Add( "R" + ( (int)( 26 + i ) ).ToString() + " : " + plcdataI[ i ].ToString() );
							}
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_plc_type_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				short data = -1;
				short result = cnc.READ_plc_type( "R", out data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( data >= 0 ) {
						string msg = "";
						switch( data ) {
							case 0:
								msg = "Byte";
								break;
							case 1:
								msg = "Short";
								break;
							case 2:
								msg = "Int";
								break;
							default:
								msg = "Unknown";
								break;
						}
						listBox3.Items.Add( "PLC R Type : " + msg );
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_plc_ver_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string data = null;
				short result = cnc.READ_plc_ver( out data );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( data != null ) {
						listBox3.Items.Add( "PLC Version : " + data );
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnREAD_nc_mem_list_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				string[][] files = null;
				short result = cnc.READ_nc_mem_list( out files );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					if( files != null ) {
						for( int i = 0; i < files.Length; i++ ) {
							listBox3.Items.Add( files[ i ][ 0 ] );
						}
					}
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnUPLOAD_nc_mem_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			OpenFileDialog ofd = new OpenFileDialog();
			if( ofd.ShowDialog() == DialogResult.OK ) {
				foreach( SyntecRemoteCNC cnc in m_CNC ) {
					short result = cnc.UPLOAD_nc_mem( ofd.FileName );
					if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
						WaitForUploadFile(cnc);
					}
					else {
						listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
					}
				}
			}
		}
		private void WaitForUploadFile(SyntecRemoteCNC cnc)
		{
			while( !cnc.isFileUploadDone ) {
				Thread.Sleep( 500 );
			}
			if( cnc.FileUploadErrorCode == (short)Syntec.Remote.SyntecRemoteCNC.ErrorCode.NormalTermination ) {
				MessageBox.Show( cnc.Host + " Upload File Success" );
			}
			else {
				MessageBox.Show( cnc.Host + " Upload File Failed" );
			}
		}
		private void btnDOWNLOAD_nc_mem_Click( object sender, EventArgs e )
		{
			if( listBox3.SelectedIndex < 0 || listBox3.Items[ listBox3.SelectedIndex ].ToString().IndexOf( "===" ) >= 0 ) {
				MessageBox.Show( "please select file" );
				return;
			}

			string host = "";
			// find host
			for( int i = listBox3.SelectedIndex - 1; i >= 0; i-- ) {
				if( listBox3.Items[ i ].ToString().IndexOf( "===" ) >= 0 ) {
					host = listBox3.Items[ i ].ToString().Replace( "===", "" );
					break;
				}
			}

			if( host == "" ) {
				MessageBox.Show( "didn't find host" );
			}

			SyntecRemoteCNC cnc = null;
			//mapping cnc
			foreach( SyntecRemoteCNC tmp in m_CNC ) {
				if( tmp.Host == host ) {
					cnc = tmp;
					break;
				}
			}

			if( cnc == null ) {
				MessageBox.Show( "didn't find cnc" );
			}

			FolderBrowserDialog fbd = new FolderBrowserDialog();
			if( fbd.ShowDialog() == DialogResult.OK ) {
				short result = cnc.DOWNLOAD_nc_mem( listBox3.Items[ listBox3.SelectedIndex ].ToString(), fbd.SelectedPath );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					WaitForDownloadFile( cnc );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void WaitForDownloadFile( SyntecRemoteCNC cnc )
		{
			while( !cnc.isFileDownloadDone ) {
				Thread.Sleep( 500 );
			}
			if( cnc.FileDownloadErrorCode == (short)Syntec.Remote.SyntecRemoteCNC.ErrorCode.NormalTermination ) {
				MessageBox.Show( cnc.Host + " Download File Success" );
			}
			else {
				MessageBox.Show( cnc.Host + " Download File Failed" );
			}
		}
		private void btnWRITE_plc_addr_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();

			if( tbWRITE_plc_addr.Text.Length <= 0 ) {
				MessageBox.Show( "fill the textbox" );
			}

			int start = 10000;
			int end = 10010;

			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				byte[] plcdataB = null;
				int[] plcdataI = null;
				short[] plcdataS = null;

				short type;

				int result = cnc.READ_plc_type( "R", out type );
				if( result == 0 ) {
					switch( type ) {
						case 0:
							plcdataB = new byte[ end - start + 1 ];
							for( int i = start; i <= end; i++ ) {
								plcdataB[ i-start ] = Convert.ToByte( tbWRITE_plc_addr.Text );
							}
							break;
						case 1:
							plcdataS = new short[ end - start + 1 ];
							for( int i = start; i <= end; i++ ) {
								plcdataS[ i - start ] = Convert.ToInt16( tbWRITE_plc_addr.Text );
							}
							break;
						case 2:
							plcdataI = new int[ end - start + 1 ];
							for( int i = start; i <= end; i++ ) {
								plcdataI[ i - start ] = Convert.ToInt32( tbWRITE_plc_addr.Text );
							}
							break;
						default:
							MessageBox.Show( "Error" );
							return;
					}

					result = cnc.WRITE_plc_addr( "R", start, end, type, plcdataB, plcdataS, plcdataI );
				}
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Done" );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnDEL_nc_mem_Click( object sender, EventArgs e )
		{
			if( listBox3.SelectedIndex < 0 || listBox3.Items[ listBox3.SelectedIndex ].ToString().IndexOf( "===" ) >= 0 ) {
				MessageBox.Show( "please select file" );
				return;
			}

			string host = "";
			// find host
			for( int i = listBox3.SelectedIndex - 1; i >= 0; i-- ) {
				if( listBox3.Items[ i ].ToString().IndexOf( "===" ) >= 0 ) {
					host = listBox3.Items[ i ].ToString().Replace( "===", "" );
					break;
				}
			}

			if( host == "" ) {
				MessageBox.Show( "didn't find host" );
			}

			SyntecRemoteCNC cnc = null;
			//mapping cnc
			foreach( SyntecRemoteCNC tmp in m_CNC ) {
				if( tmp.Host == host ) {
					cnc = tmp;
					break;
				}
			}

			if( cnc == null ) {
				MessageBox.Show( "didn't find cnc" );
			}

			short result = cnc.DEL_nc_mem( listBox3.Items[ listBox3.SelectedIndex ].ToString() );
			if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
				MessageBox.Show( cnc.Host + " :Done" );
			}
			else {
				MessageBox.Show( cnc.Host + " : Error:" + result.ToString() );
			}
		}
		private void btnREAD_nc_pointer_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				int lineno = 0;
				short result = cnc.READ_nc_pointer( out lineno );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "LineNo : " + lineno.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}

		private void button39_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				short result = cnc.WRITE_relpos( "X", Convert.ToDouble( tbWRITE_relpos.Text ) );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Write relpos Success" );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}

		private void btnIsUSBExist_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				listBox3.Items.Add( "===" + cnc.Host + "===" );
				listBox3.Items.Add( "isUSBExist: " + cnc.isUSBExist().ToString() );
			}
		}
		private void btnMainBoard_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				listBox3.Items.Add( "===" + cnc.Host + "===" );
				listBox3.Items.Add( "MainBoard: " + cnc.MainBoardPlatformName );
			}
		}
		private void btn_ReadUseTime_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {

				string Status = "";
				string TimeStart = "";
				string TimeExpire = "";
				int TimeRemain = -1;

				short result = cnc.READ_useTime( out Status, out TimeStart, out TimeExpire, out TimeRemain );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Status: " + Status );
					listBox3.Items.Add( "Time Start: " + TimeStart );
					listBox3.Items.Add( "Time Expire: " + TimeExpire );
					listBox3.Items.Add( "Time Remain: " + TimeRemain.ToString() + " hr" );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnReadRemoteTime_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {

				DateTime dt = new DateTime();

				short result = cnc.READ_remoteTime( out dt );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Remote Date Time: " + dt.ToString( "yyyy/MM/dd HH:mm:ss" ) );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnWriteRemoteDate_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				short result = cnc.WRITE_remoteDate( Convert.ToInt32( tbYear.Text ), Convert.ToInt32( tbMonth.Text ), Convert.ToInt32( tbDay.Text ) );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Write Remote Date Success!" );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnWriteRemoteTime_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {
				short result = cnc.WRITE_remoteTime( Convert.ToInt32( tbHour.Text), Convert.ToInt32( tbMinute.Text ), Convert.ToInt32( tbSecond.Text ) );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Write Remote Time Success!" );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}
		private void btnReadDiskCFreeSpace_Click( object sender, EventArgs e )
		{
			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {

				long Space = -1;

				short result = cnc.READ_diskCFreeSpace( out Space );
				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "DiskC Free Space: " + Space.ToString() );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}

		private void btnUpgrade_Click( object sender, EventArgs e )
		{
			OpenFileDialog ofd = new OpenFileDialog();
			ofd.Filter = "Software(package_*.zip)|package_*.zip|Parameter(param.dat)|param.dat|Ladder(cnc.lad)|cnc.lad";

			if( ofd.ShowDialog() != DialogResult.OK ) {
				return;
			}

			listBox3.Items.Clear();
			foreach( SyntecRemoteCNC cnc in m_CNC ) {

				short result = -1;

				string EXT = System.IO.Path.GetExtension( ofd.FileName ).ToUpper();

				switch( EXT ) {
					case ".ZIP":
						result = cnc.UPLOAD_software( ofd.FileName );
						break;
					case ".DAT":
						result = cnc.UPLOAD_param_file( ofd.FileName );
						break;
					case ".LAD":
						result = cnc.UPLOAD_plc_file( ofd.FileName );
						break;
					default:
						break;
				}

				if( result == (short)SyntecRemoteCNC.ErrorCode.NormalTermination ) {
					listBox3.Items.Add( "===" + cnc.Host + "===" );
					listBox3.Items.Add( "Upload " + System.IO.Path.GetFileName( ofd.FileName ) + " Success!" );
					listBox3.Items.Add( "Please Reboot to Upgrade!" );
				}
				else {
					listBox3.Items.Add( cnc.Host + " : Error:" + result.ToString() );
				}
			}
		}



	}

}
